/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef AP_CHANNELS_HH
#define AP_CHANNELS_HH

#include "ap-channels.hh"
#include "DatEnv.hh"
#include "OperStateCondList.hh"
#include "TSeries.hh"
#include <vector>
#include <string>
#include <iosfwd>

class channels {
public:
   //-----------------------------------  1d statistics structure
   struct chan_stat {
      chan_stat(size_t id);
      bool operator< (const chan_stat& rhs) const;
      size_t channel_id;
      size_t min_index;
      size_t trigs_found;
      double trigs_expected;
      double min_likelihood;
   };

public:
   /** Channel constructor
    * \param chan channel name
    * \param N    estimated number of elements.
    */
   channels(const std::string& chan, size_t N=32768);
   /** Channel destructor.
    */
   ~channels(void);
   /** Apend the argument time series to the end of the raw channel series.
    */
   void append(const TSeries& ts);
   std::string bin_id(size_t id) const; 
   bool empty(void) const;
   const std::string& name(void) const;
   /** prepare for next segment by removing overlapped segment.
    */
   void next(Interval dt);
   const TSeries& ref_values(void) const;
   const TSeries& ref_rank(void) const;
   int  reject(void) const;
   /** Reset the raw data series.
    */
   void reset(void);
   void set_debug(int deb);
   void set_limits(size_t M=16);
   void set_reject(int rej);
   bool test_reject(bool reject_outlier) const;
   bool test1D(const TSeries& ts, double CL, chan_stat& s, bool rej=true);

private:
   size_t      _alloc;
   std::string _name;
   TSeries     _value;
   std::vector<double> _limits;
   TSeries     _rank;
   int         _reject;
   int         _debug;
};

inline bool 
channels::empty(void) const {
   return _value.empty();
}

inline int
channels::reject(void) const {
   return _reject;
}

inline const std::string&
channels::name(void) const {
   return _name;
}

inline const TSeries&
channels::ref_values(void) const {
   return _value;
}

inline const TSeries&
channels::ref_rank(void) const {
   return _rank;
}

inline bool
channels::test_reject(bool reject_outlier) const {
   return _reject && (reject_outlier || _reject != 3);
}

#endif // !defined(AP_CHANNELS_HH)
